package co.fast.flutter.sdk

import co.fast.android.sdk.checkout.FastCheckoutEvent
import co.fast.android.sdk.checkout.Product
import co.fast.android.sdk.checkout.ProductOption

fun FastFlutter.Product.toProduct(): Product {
    return Product(
        id = this.id,
        variantId = this.variantId,
        options = this.productOptions.toProductOptions(),
        quantity = this.quantity?.toInt() ?: 0,
    )
}

fun FastFlutter.ProductOption.toProductOption(): ProductOption {
    return ProductOption(
        id = this.id.orEmpty(),
        value = this.value.orEmpty(),
    )
}

fun List<FastFlutter.ProductOption>?.toProductOptions(): List<ProductOption> {
    return this?.map { it.toProductOption() }.orEmpty()
}

fun List<FastFlutter.Product>?.toProducts(): List<Product> {
    return this?.map { it.toProduct() }.orEmpty()
}

fun FastCheckoutEvent.PurchaseInfo.toFlutterPurchaseInfo(): FastFlutter.PurchaseInfo {
    return FastFlutter.PurchaseInfo.Builder()
        .setPlatformId(platformId)
        .setRevenue(revenue)
        .setTax(tax)
        .setShipping(shipping)
        .setCurrencyCode(currencyCode)
        .setCoupon(coupon)
        .setCheckoutTime(checkoutTime?.time ?: 0L)
        .setItems(items.toFlutterPurchaseItems())
        .build()
}

fun FastCheckoutEvent.PurchaseItem.toFlutterPurchaseItem(): FastFlutter.PurchaseItem {
    return FastFlutter.PurchaseItem.Builder()
        .setPlatformId(platformId)
        .setPlatformVariantId(platformVariantId)
        .setName(name)
        .setPrice(price)
        .setQuantity(quantity.toLong())
        .setCoupon(coupon)
        .build()
}

fun List<FastCheckoutEvent.PurchaseItem>.toFlutterPurchaseItems(): List<FastFlutter.PurchaseItem> {
    return this.map { it.toFlutterPurchaseItem() }
}

fun FastCheckoutEvent.toFlutterFastCheckoutEvent(): FastFlutter.FastEvent {
    val builder = FastFlutter.FastEvent.Builder()
    when (this) {
        is FastCheckoutEvent.OrderCreated -> builder
            .setEventType(FastFlutter.FastEventType.orderCreated)
            .setPurchaseInfo(purchaseInfo.toFlutterPurchaseInfo())
        is FastCheckoutEvent.OrderUpdated -> builder
            .setEventType(FastFlutter.FastEventType.orderUpdated)
            .setPurchaseInfo(purchaseInfo.toFlutterPurchaseInfo())
        is FastCheckoutEvent.OrderCompleted -> builder
            .setEventType(FastFlutter.FastEventType.orderCompleted)
            .setPurchaseInfo(purchaseInfo.toFlutterPurchaseInfo())
        is FastCheckoutEvent.OrderCanceled -> builder
            .setEventType(FastFlutter.FastEventType.orderCanceled)
            .setPurchaseInfo(purchaseInfo.toFlutterPurchaseInfo())
        is FastCheckoutEvent.OrderFailed -> builder
            .setEventType(FastFlutter.FastEventType.orderFailed)
            .setError(throwable?.message.orEmpty())
        is FastCheckoutEvent.SDKClosed -> builder
            .setEventType(FastFlutter.FastEventType.closed)
            .setPurchaseInfo(purchaseInfo?.toFlutterPurchaseInfo())
    }

    return builder.build()
}