import 'dart:async';

import 'package:flutter/services.dart';
import 'generated_fast_api.dart';

class Fast {
  static const MethodChannel _channel = MethodChannel('fast_sdk');
  static const EventChannel _eventChannel = EventChannel(
      'co.fast.flutter.sdk/fast_events',
      StandardMethodCodec(FastFlutterApi.codec as StandardMessageCodec));

  static FastFlutterApi fastApi = FastFlutterApi(binaryMessenger: null);
  static FastEventCallbacks? _callbacks;

  static Future<void> configure(String appId) async {
    StreamSubscription subscription =
        _eventChannel.receiveBroadcastStream().listen((fastEvent) {
      String fastEventType = fastEvent.eventType?.toString() ?? "";
      String purchaseInfo = fastEvent.purchaseInfo?.toString() ?? "";
      String error = fastEvent.error?.toString() ?? "";
      print(
          "Subscription received event in dart/example: type: $fastEventType, purchaseInfo: $purchaseInfo, error: $error");

      _callbacks!(fastEvent);
    });

    // TODO cancel subscription

    return fastApi.configure(appId);
  }

  static Future<void> checkoutWithProducts(
    List<Product?> products,
    String? currencyCode,
    String? couponCode,
    FastEventCallbacks eventCallbacks,
  ) async {
    _callbacks = eventCallbacks;
    return fastApi.checkoutWithProducts(products, currencyCode, couponCode);
  }

  static Future<void> checkoutWithCartId(
    String cartId,
    String? currencyCode,
    FastEventCallbacks eventCallbacks,
  ) async {
    _callbacks = eventCallbacks;
    return fastApi.checkoutWithCartId(cartId, currencyCode);
  }

  static Future<void> reset() async {
    return fastApi.reset();
  }
}

typedef FastEventCallbacks = void Function(FastEvent event);
