import 'package:pigeon/pigeon.dart';

@HostApi()
abstract class FastFlutterApi {
  @async
  void configure(String appId);

  @async
  void reset();

  @async
  void checkoutWithProducts(
      List<Product> products, String? currencyCode, String? couponCode);

  @async
  void checkoutWithCartId(String cartId, String? currencyCode);

  @async
  void _unused(FastEvent event);
}

// Models
class Product {
  String? id;
  String? variantId;
  List<ProductOption?>? productOptions;
  int? quantity;
}

class ProductOption {
  String? id;
  String? value;
}

// Fast Checkout Events
enum FastEventType {
  orderCreated,
  orderUpdated,
  orderCompleted,
  orderCanceled,
  orderFailed,
  closed,
}

enum FastEventInfoKey {
  error,
  purchaseInfo,
}

class FastEvent {
  FastEventType? eventType;
  PurchaseInfo? purchaseInfo;
  String? error;

  FastEvent(this.eventType, this.purchaseInfo);
  FastEvent.error(this.eventType, this.error);
}

class PurchaseInfo {
  /// The ID of a purchase as represented in the seller's backend platform.
  String? platformId;

  /// The purchase total, including tax, shipping, and discounts.
  String? revenue;

  /// Total taxes for this purchase.
  String? tax;

  /// Cost of shipping for this purchase.
  String? shipping;

  /// The 3-letter ISO 4217 currency code.
  String? currencyCode;

  /// Coupon, if any, included in this purchase.
  String? coupon;

  /// Time of checkout.
  int? checkoutTime;

  /// The products included in this purchase.
  List<PurchaseItem?>? items;
}

class PurchaseItem {
  /// The ID of a product as represented in the seller's backend platform.
  String? platformId;

  /// The ID of a product variant as represented in the seller's backend platform.
  String? platformVariantId;

  /// The name of a product as displayed to the user.
  String? name;

  /// The price of a product (not including discounts, tax, etc.).
  String? price;

  /// The number of this item that was included in a purchase.
  int? quantity;

  /// Any discount applied to this product's price.
  String? coupon;
}
